function makeLoopCarProps(){
    var originalDeltaXY = cp.v(100,200);//original

    // var deltaXY = originalDeltaXY;//original

    var deltaXY = cp.v.add(originalDeltaXY,cp.v(400,-1730));//homeloop
    var prop = {
        name:"Loop",
        deltaXY:deltaXY,
        wheelSpeed:0,
        leftWheelProp:{
            radius:35,
            mass:10,
            pos:cp.v(445+deltaXY.x - 100,235+deltaXY.y-200),
            group:constCar.car.leftWheel.group,
            collisionType:constCar.car.leftWheel.collisionType,
            elastic:0.1,
            friction:0.8,

            grooveJoint:{
                anchor1:cp.v(-60, 0),
                anchor2:cp.v(-60, -40-25)
            },
            dampedSpring:{
                anchor1:cp.v(-60,-25),
                anchor2:cp.v(0,0),
                restLength:35,
                stiffness:500,
                damping:82
            }
        },
        rightWheelProp:{
            radius:35,
            mass:10,
            pos:cp.v(555+deltaXY.x - 100,235+deltaXY.y-200),
            group:constCar.car.rightWheel.group,
            collisionType:constCar.car.rightWheel.collisionType,
            elastic:0.1,
            friction:0.8,

            grooveJoint:{
                anchor1:cp.v(60, 0),
                anchor2:cp.v(60, -40-25)
            },
            dampedSpring:{
                anchor1:cp.v(60,-25),
                anchor2:cp.v(0,0),
                restLength:35,
                stiffness:800,
                damping:82
            }
        },
        chassisProp:{
            verts:[-20	,	10,
                -10	,	20,
                20	,	20,
                40	,	10,
                40	,  -10,
                -20	,  -10,
                -20	,	10],
            mass:5,
            widthHeight:cp.v(80,30),
            pos:cp.v(500+deltaXY.x - 100,300+deltaXY.y-200),
            elastic:0.5,
            friction:1,
            group:constCar.car.chassis.group,
            collisionType:constCar.car.chassis.collisionType,
            scale:2.5,
            offset:cp.v(-10,0),
            massCenterDelta:cp.v(0,0)
        },
        machineArmProp:{
            pos:cp.v(245,34)
        },
        monkey:{
            pos:cp.v(159,52.1),
            scale:0.85
        }
    };
    return prop;
}


function makeLeftWheelLoopCar(space,prop){
    var leftWheel = {};
    var radius = prop.radius;
    var mass = prop.mass;
    var pos = prop.pos;
    var group = prop.group;
    var collisionType = prop.collisionType;
    var elastic = prop.elastic;
    var friction = prop.friction;

    var body = space.addBody(new cp.Body(mass, cp.momentForCircle(mass, 0.0, radius, cp.vzero)));
    body.setPos(prop.pos);

    leftWheel.body = body;

    var shape = space.addShape(new cp.CircleShape(body, radius, cp.vzero));
    leftWheel.shape = shape;
    shape.setElasticity(elastic);
    shape.setFriction(friction);
    shape.group = group;

    shape.setCollisionType(collisionType);

    return leftWheel;
}
function makeRightWheelLoopCar(space,prop){
    var rightWheel = {};

    var radius = prop.radius;
    var mass = prop.mass;
    var pos = prop.pos;
    var group = prop.group;
    var elastic = prop.elastic;
    var friction = prop.friction;
    var collisionType = prop.collisionType;

    var body = space.addBody(new cp.Body(mass, cp.momentForCircle(mass, 0.0, radius, cp.vzero)));
    body.setPos(prop.pos);

    rightWheel.body = body;

    var shape = space.addShape(new cp.CircleShape(body, radius, cp.vzero));
    rightWheel.shape = shape;
    shape.setElasticity(elastic);
    shape.setFriction(friction);
    shape.group = group;

    shape.setCollisionType(collisionType);

    return rightWheel;
}
function makeChassisLoopCar(space,prop){
    var chassis = {};

    var verts = prop.verts;
    var mass = prop.mass;
    var pos = prop.pos;
    var elastic = prop.elastic;
    var friction = prop.friction;
    var group = prop.group;
    var collisionType = prop.collisionType;
    var scale = prop.scale;
    var offset = prop.offset;
    var widthHeight = prop.widthHeight;
    var massCenterDelta = prop.massCenterDelta;

    for(var i = 0;i < verts.length;i ++){
        if(0 == i % 2){
            verts[i] += offset.x;
        }else{
            verts[i] += offset.y;
        }
        verts[i] *= scale;
    }

    var body = space.addBody(new cp.Body(mass, cp.momentForPoly(mass, verts, cp.v(0,0))));

    body.setPos(cp.v.add(pos,massCenterDelta));

    chassis.body = body;

    var shape = space.addShape(new cp.PolyShape(body, verts, cp.vzero));
    chassis.shape = shape;

    shape.setElasticity(elastic);
    shape.setFriction(friction);
    shape.group = group;
    shape.setCollisionType(collisionType);

    return chassis;
}

function makeWheelsJointsLoopCar(space,chassis,leftWheel,rightWheel,leftWheelProp,rightWheelProp,carprop){
    //grooveJoint
    leftWheel.grooveJoint = makeGrooveJoint(space,chassis.body, leftWheel.body, leftWheelProp.grooveJoint.anchor1, leftWheelProp.grooveJoint.anchor2);
    rightWheel.grooveJoint = makeGrooveJoint(space,chassis.body, rightWheel.body, rightWheelProp.grooveJoint.anchor1, rightWheelProp.grooveJoint.anchor2);
    //dampedSpring
    leftWheel.dampedSpring = makeDampedSpring(space,chassis.body,leftWheel.body,leftWheelProp.dampedSpring.anchor1,leftWheelProp.dampedSpring.anchor2,
        leftWheelProp.dampedSpring.restLength,leftWheelProp.dampedSpring.stiffness,leftWheelProp.dampedSpring.damping);
    rightWheel.dampedSpring = makeDampedSpring(space,chassis.body,rightWheel.body,rightWheelProp.dampedSpring.anchor1,rightWheelProp.dampedSpring.anchor2,
        rightWheelProp.dampedSpring.restLength,rightWheelProp.dampedSpring.stiffness,rightWheelProp.dampedSpring.damping);
    //motorJoint
    leftWheel.motorJoint = makeMotorJoint(space,leftWheel.body,chassis.body,carprop.wheelSpeed);
    rightWheel.motorJoint = makeMotorJoint(space,chassis.body,rightWheel.body,carprop.wheelSpeed*(-1));
}

function makeMachineArmLoopCar(parent,pos){
    var machinearm = {};
    machinearm.animName = ["cycle","land-3","land-5","land-1-4","land-2-qiqiu","stone-1","stone-2","sea-zhangyu","sea-xiong","sea-jinyu","sea-haima"];
    machinearm = createSkeletonSprite("res/car_a/common/che/anim/skeleton.json", "res/car_a/common/che/anim/skeleton.atlas", 1, parent, pos, -1);

    machinearm.setCompleteListener(function(trackEntry){
        var animationName = trackEntry.animation ? trackEntry.animation.name : "nullname";
        if(animationName != "cycle") {
            mylog(animationName + " is over");
        }
        if("nullname" != animationName && "cycle" != animationName){
            machinearm.setRotation(0);
            machinearm.setAnimation(1,"cycle",true);
            machinearm.runAction(cc.repeatForever(cc.sequence(cc.rotateTo(1,2),cc.rotateTo(1,-4),cc.rotateTo(1,2))));
        }
    });

    machinearm.runCycle = function(){
        machinearm.runAction(cc.repeatForever(cc.sequence(cc.rotateTo(1,2),cc.rotateTo(1,-4),cc.rotateTo(1,2))));
        machinearm.setAnimation(1,"cycle",true);
    }

    machinearm.runCycle();
    return machinearm;
}
function makeMonkeyLoopCar(parent,pos,scale){
    var skel = {};

    skel.animName = ["biaoqing1","biaoqing2","biaoqing3","biaoqing4","biaoqing5"];
    skel.skelSprite = createSkeletonSprite("res/skel/monkey/skeleton.json", "res/skel/monkey/skeleton.atlas", scale, parent, pos, -1);
    skel.skelSprite.setGlobalZOrder(constCar.globalZOrder.level_pos5);

    skel.skelSprite.setCompleteListener(function(trackEntry){
        // var animationName = trackEntry.animation ? trackEntry.animation.name : "nullname";
        // if(animationName != "cycle") {
        //     mylog(animationName + " is over");
        // }
        // if("nullname" != animationName && "cycle" != animationName){
        //     machinearm.setRotation(0);
        //     machinearm.setAnimation(1,"cycle",true);
        //     machinearm.runAction(cc.repeatForever(cc.sequence(cc.rotateTo(1,2),cc.rotateTo(1,-4),cc.rotateTo(1,2))));
        // }
    });

    skel.runCycle = function(){
        // skel.skelSprite.runAction(cc.repeatForever(cc.sequence(cc.rotateTo(1,2),cc.rotateTo(1,-4),cc.rotateTo(1,2))));
        skel.skelSprite.setAnimation(1,skel.animName[1],true);
    }

    skel.runCycle();
    return skel;
}


function makeLoopCar(space,carProp,parent){
    var car = {};
    car.balance_sin = 0.0;
    car.last_v = 0.0;

    car.prop = carProp;
    car.leftWheel = makeLeftWheelLoopCar(space,car.prop.leftWheelProp);
    car.rightWheel = makeRightWheelLoopCar(space,car.prop.rightWheelProp);
    car.chassis = makeChassisLoopCar(space,car.prop.chassisProp);

    //
    car.chassisSpr = makeSkin(car.chassis,"res/car_a/common/che/chesheng",cc.p(0.54,0.18),parent,0,0,cc.p(400+car.prop.deltaXY.x,100+car.prop.deltaXY.y),1);
    car.leftWheelSpr = makeSkin(car.leftWheel,"res/car_a/common/che/chelun1",cc.p(0.5,0.5),parent,0,0,cc.p(345+car.prop.deltaXY.x,35+car.prop.deltaXY.y),1);
    car.rightWheelSpr = makeSkin(car.rightWheel,"res/car_a/common/che/chelun1",cc.p(0.5,0.5),parent,0,0,cc.p(455+car.prop.deltaXY.x,35+car.prop.deltaXY.y),1);


    car.leftSpr = new cc.Sprite("res/car_a/common/che/chelun1");
    car.leftSpr.setGlobalZOrder(constCar.globalZOrder.level_pos10);
    car.chassisSpr.addChild(car.leftSpr);
    car.leftSpr.setPosition(cp.v(76.5,-20));

    car.rightSpr = new cc.Sprite("res/car_a/common/che/chelun1");
    car.rightSpr.setGlobalZOrder(constCar.globalZOrder.level_pos10);
    car.chassisSpr.addChild(car.rightSpr);
    car.rightSpr.setPosition(cp.v(76.5+120.5,-20));

    car.isVisibleLeftRightSpr = function(isVisible){
        mylog("isVisibleLeftRightSpr false");

        car.leftSpr.setVisible(isVisible);
        car.rightSpr.setVisible(isVisible);

        car.leftWheelSpr.setVisible(!isVisible);
        car.rightWheelSpr.setVisible(!isVisible);
    };

    car.machinearm = makeMachineArmLoopCar(car.chassisSpr,car.prop.machineArmProp.pos);
    car.machinearm.setGlobalZOrder(constCar.globalZOrder.level_pos10);

    car.monkeySkel = makeMonkeyLoopCar(car.chassisSpr,car.prop.monkey.pos,car.prop.monkey.scale);

    makeWheelsJointsLoopCar(space,car.chassis,car.leftWheel,car.rightWheel,car.prop.leftWheelProp,car.prop.rightWheelProp,car.prop);

    car.addLeftCollisionLine = function(space,body,prop){
        var group = prop.group;
        var width = prop.width;
        var layers = prop.layers;

        var shape = space.addShape(new cp.SegmentShape(body,cp.v(-120,-70),cp.v(-120,70),width));
        shape.group = group;
        car.leftcollisionshape = shape;
        // shape.layers = ~cp.ALL_LAYERS;//because of default layers is cp.ALL_LAYERS
        // shape.layers = ~constCar.collisionlines.layers;
        // shape.layers = 0xfffe;
        shape.layers = layers;

        return shape;
    };
    car.addLeftCollisionLine(space,car.chassis.body,{
        group:parent.metaData.road.group,
        width:5,
        layers:~constCar.collisionlines.layers
    });

    car.addRightCollisionLine = function(space,body,prop){
        var group = prop.group;
        var width = prop.width;
        var layers = prop.layers;

        var shape = space.addShape(new cp.SegmentShape(body,cp.v(120,-70),cp.v(120,70),width));
        shape.group = group;
        car.rightcollisionshape = shape;
        // shape.layers = ~cp.ALL_LAYERS;//because of default layers is cp.ALL_LAYERS
        // shape.layers = ~constCar.collisionlines.layers;
        // shape.layers = 0xfffe;
        shape.layers = layers;

        return shape;
    };
    car.addRightCollisionLine(space,car.chassis.body,{
        group:parent.metaData.road.group,
        width:5,
        layers:~constCar.collisionlines.layers
    });

    // car.leftWheelSpr.setOpacity(100);
    // car.rightWheelSpr.setOpacity(100);
    // car.chassisSpr.setOpacity(100);
    // car.machinearm.setOpacity(150);

    car.addPuff = function(par){
        car.puff = {};

        car.puff.particleSystem = new cc.ParticleSystem("res/common/huoyan/puff.plist");
        car.puff.particleSystem.setGlobalZOrder(par.getGlobalZOrder());
        // car.puff.particleSystem.duration = cc.ParticleSystem.DURATION_INFINITY;
        // car.puff.particleSystem.setAutoRemoveOnFinish(true);
        car.puff.particleSystem.setPosition(cp.v(36,120));
        // car.puff.particleSystem.setPositionType(cc.ParticleSystem.TYPE_GROUPED);
        par.addChild(car.puff.particleSystem, -10);


        return car.puff;
    }(car.chassisSpr);

    car.audioManager = function(){
        var audioManager = {};
        audioManager.carAudioRunningCtl = audioController("res/common/audio/car/xingzou.mp3",true,constCar.volume.maxVolume,constCar.volume.minVolume,function(){

        });

        audioManager.carAudioRunningCtl.start();
        audioManager.carAudioRunningCtl.setVolume(0);

        return audioManager;
    }();

    //first : this.car.leftWheel.motorJoint.rate = -10;//change speed
    //second: set the gravity to 0,then by the physicssprite run action to change the speed or the run line
    car.changeSpeed = function(speed){
        car.prop.wheelSpeed = speed;
        car.leftWheel.motorJoint.rate = car.prop.wheelSpeed * (-1);
        car.rightWheel.motorJoint.rate = car.prop.wheelSpeed * 1;
    };

    car.stopToTargetX = function(dt,target_x,callback,idx,updatecontroller) {
        stopToTargetX(dt, target_x, car, callback, idx, updatecontroller);
    }

    car.resetAllForces = function(){};

    car.removeAllJointsShapeBody = function(){
        space.removeConstraint(car.leftWheel.grooveJoint);
        space.removeConstraint(car.rightWheel.grooveJoint);

        space.removeConstraint(car.leftWheel.dampedSpring);
        space.removeConstraint(car.rightWheel.dampedSpring);

        space.removeConstraint(car.leftWheel.motorJoint);
        space.removeConstraint(car.rightWheel.motorJoint);

        space.removeShape(car.leftWheel.shape);
        space.removeShape(car.rightWheel.shape);
        space.removeShape(car.chassis.shape);

        space.removeShape(car.leftcollisionshape);
        space.removeShape(car.rightcollisionshape);

        space.removeBody(car.leftWheel.body);
        space.removeBody(car.rightWheel.body);
        space.removeBody(car.chassis.body);

        car.resetAllForces();
    };

    car.readdAllJointsShapeBody = function(deltaxy){
        car.moveDeltaXY = deltaxy;
        //我一直不明白的是chassisbody和wheelbody的处理方式不同
        var leftwheelbody = car.leftWheel.body;
        leftwheelbody.setPos(cp.v.add(leftwheelbody.getPos(),cp.v.add(deltaxy,cp.v(0,0))));
        space.addBody(leftwheelbody);

        var rightwheelbody = car.rightWheel.body;
        rightwheelbody.setPos(cp.v.add(rightwheelbody.getPos(),cp.v.add(deltaxy,cp.v(0,0))));
        space.addBody(rightwheelbody);


        var chassisbody = car.chassis.body;
        chassisbody.setPos(cp.v.add(chassisbody.getPos(),cp.v.add(cp.v(0,0),cp.v(0,0))));
        space.addBody(chassisbody);

        space.addShape(car.leftWheel.shape);
        space.addShape(car.rightWheel.shape);
        space.addShape(car.chassis.shape);

        space.addShape(car.leftcollisionshape);
        space.addShape(car.rightcollisionshape);

        space.addConstraint(car.leftWheel.grooveJoint);
        space.addConstraint(car.rightWheel.grooveJoint);

        space.addConstraint(car.leftWheel.dampedSpring);
        space.addConstraint(car.rightWheel.dampedSpring);

        space.addConstraint(car.leftWheel.motorJoint);
        space.addConstraint(car.rightWheel.motorJoint);

        car.resetAllForces();
    };

    car.readdAllJointsShapeBodyNormal = function(){
        var leftwheelbody = car.leftWheel.body;
        leftwheelbody.setPos(cp.v.add(leftwheelbody.getPos(),cp.v(0,0)));
        space.addBody(leftwheelbody);

        var rightwheelbody = car.rightWheel.body;
        rightwheelbody.setPos(cp.v.add(rightwheelbody.getPos(),cp.v(0,0)));
        space.addBody(rightwheelbody);


        var chassisbody = car.chassis.body;
        chassisbody.setPos(cp.v.add(chassisbody.getPos(),cp.v(0,0)));
        space.addBody(chassisbody);

        space.addShape(car.leftWheel.shape);
        space.addShape(car.rightWheel.shape);
        space.addShape(car.chassis.shape);

        space.addShape(car.leftcollisionshape);
        space.addShape(car.rightcollisionshape);

        space.addConstraint(car.leftWheel.grooveJoint);
        space.addConstraint(car.rightWheel.grooveJoint);

        space.addConstraint(car.leftWheel.dampedSpring);
        space.addConstraint(car.rightWheel.dampedSpring);

        space.addConstraint(car.leftWheel.motorJoint);
        space.addConstraint(car.rightWheel.motorJoint);

        car.resetAllForces();
    };


    car.isVisibleLeftRightSpr(false);



    car.destroySelf = function(){
        car.removeAllJointsShapeBody();

        car.chassisSpr.removeFromParent(true);
        car.leftWheelSpr.removeFromParent(true);
        car.rightWheelSpr.removeFromParent(true);
    }
    return car;
}
